/*!
    \file    main.c
    \brief   HASH SHA224 SHA256 dma example

    \version 2025-01-16, V1.4.0, firmware for GD32VW55x
*/

/*
    Copyright (c) 2025, GigaDevice Semiconductor Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

#include <string.h>
#include "gd32vw553h_eval.h"
#include "main.h"

/* hash SHA224/SHA256 digest computation */
static void hash_sha224_sha256_dma(uint32_t algo);
/* printf data in words */
static void data_display(uint32_t datalength);
/* check data in bytes */
static void data_check(uint8_t *src, uint8_t *dst, uint32_t len);

uint32_t i = 0, len = 0;
hau_digest_parameter_struct   digest_para;
__IO uint8_t check_flag = 0;

/*!
    \brief      main function
    \param[in]  none
    \param[out] none
    \retval     none
*/
int main(void)
{
    gd_eval_led_init(LED1);
    gd_eval_com_init(EVAL_COM0);
    /* enable HAU clock */
    rcu_periph_clock_enable(RCU_HAU);
    /* enable DMA1 clock */
    rcu_periph_clock_enable(RCU_DMA);

    len = sizeof(message_input);
    printf("\r\n message to be hashed:\r\n\r\n");
    for(i = 0; i < len ; i++){
        printf("%c", message_input[i]);
    }

    /* SHA-224 digest computation */
    hash_sha224_sha256_dma(HAU_ALGO_SHA224);
    printf("  \r\n\r\nSHA224 message digest (224 bits):\r\n\r\n");
    data_display(28U);
    data_check((uint8_t *)&digest_para.out[0], (uint8_t *)&expected_data_sha224[0], 28U);

    /* SHA-256 digest computation */
    hash_sha224_sha256_dma(HAU_ALGO_SHA256);
    printf("  \r\n\r\nSHA256 message digest (256 bits):\r\n\r\n");
    data_display(32U);
    data_check((uint8_t *)&digest_para.out[0], (uint8_t *)&expected_data_sha256[0], 32U);

    if(0U == check_flag) {
        gd_eval_led_on(LED1);
    }

    while(1) {
    }
}

/*!
    \brief      hash SHA224/SHA256 digest computation
    \param[in]  algo: algorithm direction
      \arg        HAU_ALGO_SHA224
      \arg        HAU_ALGO_SHA256
    \param[out] none
    \retval     none
*/
static void hash_sha224_sha256_dma(uint32_t algo)
{
    dma_multi_data_parameter_struct dma_init_parameter;
    hau_init_parameter_struct hau_initpara;
    __IO uint16_t num_last_valid = 0;
    __IO uint32_t meg_burst = 0;

    /* configure HAU */
    hau_deinit();
    hau_init_struct_para_init(&hau_initpara);
    hau_initpara.algo = algo;
    hau_initpara.mode = HAU_MODE_HASH;
    hau_initpara.datatype = HAU_SWAPPING_8BIT;
    hau_init(&hau_initpara);

    if(len % 4) {
        meg_burst++;
    }
    meg_burst += (len / 4);
    num_last_valid = 8 * (len % 4);
    /* configure the number of valid bits in last word of the message */
    hau_last_word_validbits_num_config(num_last_valid);

    /* configure DMA */
    dma_deinit(DMA_CH7);
    dma_multi_data_para_struct_init(&dma_init_parameter);
    dma_init_parameter.periph_addr = (uint32_t)(&HAU_DI);
    dma_init_parameter.periph_width = DMA_PERIPH_WIDTH_32BIT;
    dma_init_parameter.periph_inc = DMA_PERIPH_INCREASE_DISABLE;
    dma_init_parameter.memory0_addr = (uint32_t)message_input;
    dma_init_parameter.memory_width = DMA_MEMORY_WIDTH_32BIT;
    dma_init_parameter.memory_inc = DMA_MEMORY_INCREASE_ENABLE;
    dma_init_parameter.memory_burst_width = DMA_MEMORY_BURST_SINGLE;
    dma_init_parameter.periph_burst_width = DMA_MEMORY_BURST_SINGLE;
    dma_init_parameter.critical_value = DMA_FIFO_4_WORD;
    dma_init_parameter.circular_mode = DMA_CIRCULAR_MODE_DISABLE;
    dma_init_parameter.direction = DMA_MEMORY_TO_PERIPH;
    dma_init_parameter.number = meg_burst;
    dma_init_parameter.priority = DMA_PRIORITY_ULTRA_HIGH;
    dma_multi_data_mode_init(DMA_CH7, &dma_init_parameter);

    dma_channel_subperipheral_select(DMA_CH7, DMA_SUBPERI2);

    /* enable DMA channel */
    dma_channel_enable(DMA_CH7);

    hau_dma_enable();

    /* wait until the last transfer from OUT FIFO */
    while(!dma_flag_get(DMA_CH7, DMA_FLAG_FTF)) {
    }
    dma_flag_clear(DMA_CH7, DMA_FLAG_FTF);
    /* wait until the busy flag is reset */
    while(hau_flag_get(HAU_FLAG_BUSY)) {
    }

    hau_digest_read(&digest_para);

    /* change formatting */
    digest_para.out[0] = __REV(digest_para.out[0]);
    digest_para.out[1] = __REV(digest_para.out[1]);
    digest_para.out[2] = __REV(digest_para.out[2]);
    digest_para.out[3] = __REV(digest_para.out[3]);
    digest_para.out[4] = __REV(digest_para.out[4]);
    digest_para.out[5] = __REV(digest_para.out[5]);
    digest_para.out[6] = __REV(digest_para.out[6]);
    digest_para.out[7] = __REV(digest_para.out[7]);
}

/*!
    \brief      printf data in words
    \param[in]  datalength: length of the data to display
    \param[out] none
    \retval     none
*/
static void data_display(uint32_t datalength)
{
    uint32_t i =0U;
    uint32_t count = 0U;
    uint8_t *temp = (uint8_t *)&digest_para.out;

    for(i = 0; i < datalength; i++){
        printf(" 0x%02X", (uint8_t)temp[i]);
        count++;

        if(4U == count){
            count = 0U;
            printf("\r\n");
        }
    }
}

/*!
    \brief      check data in bytes
    \param[in]  src: source
    \param[in]  dst: destination
    \param[in]  len: length of the data to check
    \param[out] none
    \retval     none
*/
static void data_check(uint8_t *src, uint8_t *dst, uint32_t len)
{
    if(memcmp((const void*)src, (const void*)dst, len) != 0){
        check_flag++;
    }
}

/* retarget the C library printf function to the USART */
int _put_char(int ch)
{
    usart_data_transmit(USART0, (uint8_t) ch);
    while(usart_flag_get(USART0, USART_FLAG_TBE) == RESET) {
    }

    return ch;
}
